/*
 * HAL.c
 *
 * Created: 09.09.2020 20:35:11
 *  Author: GoldenAndy
 */ 

#include "HAL.h"
#include "../_common/i2c_dumb_master/i2c_mstr.h"

//------------------------------------------- Display

	#define MATRIX_PORT				B
	#define MATRIX_CS_BITVAL		_BV(PORTB2)
	#define MATRIX_DATA_BITVAL		_BV(PORTB3)
	#define MATRIX_CLK_BITVAL		_BV(PORTB5)
	
	#define USED_PORTB_MATRIX		( MATRIX_CS_BITVAL | MATRIX_DATA_BITVAL | MATRIX_CLK_BITVAL )

	void matrixSetReg(uint8_t regAddr, uint8_t regData) {
		SPDR = regAddr;
		while(!(SPSR & (1<<SPIF)));
		SPDR = regData;
		while(!(SPSR & (1<<SPIF)));
	}

	void matrixSetCsLevel(uint8_t csLevel){
		if (csLevel) __OUTPORT(MATRIX_PORT) |= MATRIX_CS_BITVAL;
		else __OUTPORT(MATRIX_PORT) &= ~MATRIX_CS_BITVAL;
	}
	
	static void matrixInit(void){
		__DDRPORT(MATRIX_PORT) |= MATRIX_CS_BITVAL | MATRIX_DATA_BITVAL | MATRIX_CLK_BITVAL;
		__OUTPORT(MATRIX_PORT) |= MATRIX_CS_BITVAL;
		SPCR = _BV(SPE) | _BV(MSTR);
		SPSR = _BV(SPI2X);
	}

	
//------------------------------------------- Buzzer
	// Buzzer on PORTB1 (Timer 1 - OC1A)
	#define BUZZER_PORT				B
	#define BUZZER_BITVAL			_BV(PORTB1)
	
	#define USED_PORTB_SOUND		BUZZER_BITVAL
	
	void soundOn(void){
		TCCR1A |= _BV(COM1A1);
	}

	void soundOff(void){
		TCCR1A &= ~( _BV(COM1A1) | _BV(COM1A0) );
		__OUTPORT(BUZZER_PORT) &= ~BUZZER_BITVAL;
	}
	
	static void soundInit(void){
		// FastPWM mode, TOP on ICR1 : WGM1[3:0] = 1110
		// ICR1 (TOP) = Clk / Sound freq
		// Divider = 1 : CS1[2:0] = 001
		TCCR1A = ( 1 << WGM11 ) | ( 0 << WGM10 );
		TCCR1B = ( 1 << WGM13 ) | ( 1 << WGM12 ) |	( 0 << CS12 ) |	( 0 << CS11 ) |	( 1 << CS10 );
		ICR1 = (F_CPU + SOUND_FREQ/2) / SOUND_FREQ  - 1 ;
		OCR1A = ((F_CPU + SOUND_FREQ/2) / SOUND_FREQ  - 1 ) / 2;
		__OUTPORT(BUZZER_PORT) &= ~BUZZER_BITVAL;
		__DDRPORT(BUZZER_PORT) |=  BUZZER_BITVAL;
	}

//------------------------------------------- External DS18B20  

	// 2: External 1-Wire : PORTB7
	
	#define ONEWIRE_PORT			B
	#define ONEWIRE_EXT_BITVAL		_BV(PORTB7)	 
	
	#define USED_PORTB_ONEWIRE		( ONEWIRE_EXT_BITVAL )

	
	void owPullUp(void)		{
		__OUTPORT(ONEWIRE_PORT) |=  ONEWIRE_EXT_BITVAL;
		__DDRPORT(ONEWIRE_PORT) |=  ONEWIRE_EXT_BITVAL;
	}
	void owPullDown(void)		{
		__OUTPORT(ONEWIRE_PORT) &= ~ONEWIRE_EXT_BITVAL;
		__DDRPORT(ONEWIRE_PORT) |=  ONEWIRE_EXT_BITVAL;
		}
		
	void owRelease(void)		{
		__DDRPORT(ONEWIRE_PORT) &= ~ONEWIRE_EXT_BITVAL;
		__OUTPORT(ONEWIRE_PORT) &= ~ONEWIRE_EXT_BITVAL;
		}
		
	uint8_t owGetLevel(void)	{
		return (__INPORT(ONEWIRE_PORT) & ONEWIRE_EXT_BITVAL) > 0;
		}
 
 	static void oneWireInit(void) {
		 __DDRPORT(ONEWIRE_PORT) &= ~( ONEWIRE_EXT_BITVAL );
		 __OUTPORT(ONEWIRE_PORT) &= ~( ONEWIRE_EXT_BITVAL );
	 }

//------------------------------------------- KeyBoard
	// Key 1 - (Sel) - PORTC0
	// Key 2 - ( - ) - PORTC1
	// Key 3 - ( + ) - PORTC2
	// Key 4 - (Ok ) - PORTC3
	
	#define KEY_PORT				C
	// BitVal-   HAL.h - KEY_MASK_SEL, KEY_MASK_DEC, KEY_MASK_INC, KEY_MASK_OK
	#define USED_PORTC_KEYBOARD		( KEY_MASK_SEL | KEY_MASK_DEC | KEY_MASK_INC | KEY_MASK_OK )

	uint8_t keyGetPress(void) {
		return (~__INPORT(KEY_PORT)) & ( KEY_MASK_SEL | KEY_MASK_DEC | KEY_MASK_INC | KEY_MASK_OK );
	}
	
	static void keyInit(void){
		__DDRPORT(KEY_PORT) &= ~( KEY_MASK_SEL | KEY_MASK_DEC | KEY_MASK_INC | KEY_MASK_OK );
		__OUTPORT(KEY_PORT) |=  ( KEY_MASK_SEL | KEY_MASK_DEC | KEY_MASK_INC | KEY_MASK_OK );
	}

//------------------------------------------- LightSensor
	// Light sensor on ADC6, AREF connected to capacitor
	// REFS[1:0] = 01
	// ADLAR = 1 - for filtering noise - set left adjusting result value (use only high byte as result)
	// MUX[3:0] = 0110
	static void lightSensorInit(void){
		ADMUX = ( 0 << REFS1 ) | ( 1 << REFS0 ) \
				| ( 1 << ADLAR ) 				\
				| ( 0 << MUX3 ) | ( 1 << MUX2 ) | ( 1 << MUX1 ) | ( 0 << MUX0 );
		ADCSRA = ( 1 << ADEN ) | ( 1 << ADSC )  \
				| ( 1 << ADPS2 ) | ( 1 << ADPS1 ) | ( 0 << ADPS0 );
		while (!(ADCSRA & _BV(ADIF))) {} //   
		ADCSRA |= _BV(ADSC) | _BV(ADIF); //   
	} 

	uint8_t lightSensorGetLevel(void) {
		uint16_t res = ADC;
		ADCSRA |= _BV(ADSC);
		return res>>8;
	}

//-------------------------------------------  DS3231 
	// i2c.SDA - PORTC4
	// i2c.SCL - PORTC5
	// 1Hz int - PORTD3 - Int1
	#define IIC_PORT			C
	#define IIC_SDA_BITVAL		_BV(PORTC4)
	#define IIC_SCL_BITVAL		_BV(PORTC5)
	#define RTC_IRQ_PORT		D
	#define RTC_IRQ_BITVAL		_BV(PORTD3)
	
	#define USED_PORTC_RTC		( IIC_SDA_BITVAL | IIC_SCL_BITVAL )
	#define USED_PORTD_RTC		RTC_IRQ_BITVAL
	
	static void rtcInit(void){
		__DDRPORT(IIC_PORT) &= ~( IIC_SDA_BITVAL | IIC_SCL_BITVAL );
		__OUTPORT(IIC_PORT) &= ~( IIC_SDA_BITVAL | IIC_SCL_BITVAL );
		__DDRPORT(RTC_IRQ_PORT) &= ~RTC_IRQ_BITVAL;
		__OUTPORT(RTC_IRQ_PORT) |=  RTC_IRQ_BITVAL;
		iicInit(0x02, 0x00);//400 kHz
		//int1
		GIFR |= _BV(INTF1);
		MCUCR &= ~_BV(ISC11); // ISC1[1:0] = 01 - any level
		MCUCR |=  _BV(ISC10);
		GICR |= _BV(INT1);
	}
	
	uint8_t getRtc1HzLevel(void){
		return __INPORT(RTC_IRQ_PORT) & RTC_IRQ_BITVAL;
	}
	
//------------------------------------------- System clock 
	// timer 2
	static void initSystemTimer(void){
		// WGM2[1:0] = 10 - CTC mode (TOP in OCR2)
		TCNT2 = 0;
		// CS2[2:0] = 100 - /64
		// F = 8000000 / (K=125*64) = 1000 Hz
		// K = F_CPU / (1000*64)
		OCR2 = (F_CPU + 1000ul*32 ) / (1000ul * 64)-1;
		TCCR2 = ( 1 << WGM21 ) | ( 0 << WGM20 )
				| ( 1 << CS22 ) | ( 0 << CS21 ) | ( 0 << CS20 );
		TIMSK |= _BV(OCIE2);
	}




//------------------------------------------- Hardware init 
	void hardwareInit(void){
		matrixInit();
		soundInit();
		oneWireInit();
		keyInit();
		lightSensorInit();
		rtcInit();
		initSystemTimer();
		
		//       
		DDRB &=   ( USED_PORTB_MATRIX | USED_PORTB_SOUND | USED_PORTB_ONEWIRE );
		PORTB |= ~( USED_PORTB_MATRIX | USED_PORTB_SOUND | USED_PORTB_ONEWIRE );
 		DDRC &=   ( USED_PORTC_KEYBOARD | USED_PORTC_RTC );
 		PORTC |= ~( USED_PORTC_KEYBOARD | USED_PORTC_RTC );
 		DDRD &=   ( USED_PORTD_RTC );
 		PORTD |= ~( USED_PORTD_RTC );
		
		//    
		SFIOR &= ~_BV(ACME);//  
		ACSR = _BV(ACD);
	}
