/*
 * matrix.c
 *
 * Created: 03.09.2020 18:00:45
 *  Author: GoldenAndy
 */ 												   

#include "matrix.h"
#include "../_common/random/random.h"
#include <string.h>
#include <avr/pgmspace.h>

#define MREG_TESTMODE		0x0F
#define MREG_STANDBY		0x0C
#define MREG_SCANLIMIT		0x0B
#define MREG_INTENSITY		0x0A
#define MREG_DECODING		0x09

#define ROLLING_DELAY		25
#define WATERFALL_DELAY		40
#define VROLLING_DELAY		60



static struct {
	matrixUpdateType	update;
	uint8_t				videoRam[32];
	uint8_t				shadowRam[32];
	uint8_t				intensity;
	} matrix;
	
	
static struct {
	uint8_t*			buff;
	const uint8_t*		fontpos;
	uint8_t				colCount;
	uint8_t				pause;
	uint8_t				tail;
	uint8_t				tailStop;
} effectsData;
	
	
	// 
		/*
		 
		    + 1 
		   -   ASCII,  -  
		    ASCII- ==0
		      
		   
	*/
	#define MAINFONT_WIDTH		6
	
	const uint8_t mainFont[] PROGMEM = {
		0x20,		0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	/* Char code = 0x20 (32), Char =  */
		0x27,		0x00, 0x00, 0xD0, 0xE0, 0x00, 0x00,	/* Char code = 0x27 (39), Char = '*/
		//0x2B,		0x00, 0x08, 0x08, 0x3E, 0x08, 0x08,	/* Char code = 0x2B (43), Char = +*/
		0x2C,		0x00, 0x00, 0x06, 0x07, 0x00, 0x00,	/* Char code = 0x2C (44), Char = ,*/
		0x2D,		0x00, 0x08, 0x08, 0x08, 0x08, 0x00,	/* Char code = 0x2D (45), Char = -*/
		0x2E,		0x00, 0x00, 0x06, 0x06, 0x00, 0x00,	/* Char code = 0x2E (46), Char = .*/
		0x30,		0x7E, 0xFF, 0x81, 0x81, 0xFF, 0x7E,	/* Char code = 0x30 (48), Char = 0*/
		0x31,		0x00, 0x21, 0xFF, 0xFF, 0x01, 0x00,	/* Char code = 0x31 (49), Char = 1*/
		0x32,		0x43, 0xC7, 0x85, 0x89, 0xF9, 0x71,	/* Char code = 0x32 (50), Char = 2*/
		0x33,		0x42, 0xC3, 0x81, 0x91, 0xFF, 0x6E,	/* Char code = 0x33 (51), Char = 3*/
		0x34,		0xF8, 0xF8, 0x08, 0x08, 0xFF, 0xFF,	/* Char code = 0x34 (52), Char = 4*/
		0x35,		0xF2, 0xF3, 0x91, 0x91, 0x9F, 0x8E,	/* Char code = 0x35 (53), Char = 5*/
		0x36,		0x7E, 0xFF, 0x91, 0x91, 0x9F, 0x0E,	/* Char code = 0x36 (54), Char = 6*/
		0x37,		0xC0, 0xC0, 0x80, 0x8F, 0xFF, 0xF0,	/* Char code = 0x37 (55), Char = 7*/
		0x38,		0x6E, 0xFF, 0x91, 0x91, 0xFF, 0x6E,	/* Char code = 0x38 (56), Char = 8*/
		0x39,		0x70, 0xF9, 0x89, 0x89, 0xFF, 0x7E,	/* Char code = 0x39 (57), Char = 9*/
		0x3A,		0x00, 0x00, 0x66, 0x66, 0x00, 0x00,	/* Char code = 0x3A (58), Char = :*/
		0x7F,		0x70, 0xF8, 0x88, 0xF8, 0x70, 0x00,	/* Char code = 0x7F (127)           */
		0xB2,		0x00, 0x81, 0xFF, 0xFF, 0x81, 0x00,	/* Char code = 0xB2 (178), Char = */
		
		0xC0,		0x7F, 0xFF, 0x88, 0x88, 0xFF, 0xFF,	/* Char code = 0xC0 (192), Char = */
		0xC1,		0xFF, 0xFF, 0x91, 0x91, 0x9F, 0xCE,	/* Char code = 0xC1 (193), Char = */
		0xC2,		0xFF, 0xFF, 0x91, 0x91, 0xFF, 0x6E,	/* Char code = 0xC2 (194), Char = */
		0xC3,		0xFF, 0xFF, 0x80, 0x80, 0xC0, 0xC0,	/* Char code = 0xC3 (195), Char = */
		0xC4,		0x03, 0x7F, 0x82, 0xFE, 0xFF, 0x03,	/* Char code = 0xC4 (196), Char = */
		0xC5,		0xFF, 0xFF, 0x89, 0x89, 0xC1, 0xC1,	/* Char code = 0xC5 (197), Char = */
		0xC6,		0xF7, 0x08, 0xFF, 0xFF, 0x08, 0xF7,	/* Char code = 0xC6 (198), Char = */
		0xC8,		0xFF, 0xFF, 0x06, 0x18, 0xFF, 0xFF,	/* Char code = 0xC8 (200), Char = */
		0xC9,		0xFF, 0x3F, 0x84, 0x88, 0x3F, 0xFF,	/* Char code = 0xC9 (201), Char = */
		0xCA,		0xFF, 0xFF, 0x18, 0x28, 0x4F, 0xC7,	/* Char code = 0xCA (202), Char = */
		0xCB,		0x01, 0x7F, 0xFE, 0x80, 0xFF, 0xFF,	/* Char code = 0xCB (203), Char = */
		0xCC,		0xFF, 0x60, 0x1C, 0x1C, 0x60, 0xFF,	/* Char code = 0xCC (204), Char = */
		0xCD,		0xFF, 0xFF, 0x08, 0x08, 0xFF, 0xFF,	/* Char code = 0xCD (205), Char = */
		0xCE,		0x7E, 0xFF, 0x81, 0x81, 0xFF, 0x7E,	/* Char code = 0xCD (206), Char = */
		0xCF,		0xFF, 0xFF, 0x80, 0x80, 0xFF, 0xFF,	/* Char code = 0xCF (207), Char = */
		0xD0,		0xFF, 0xFF, 0x88, 0x88, 0xF8, 0x70,	/* Char code = 0xD0 (208), Char = */
		0xD1,		0x7E, 0xFF, 0x81, 0x81, 0xC3, 0x42,	/* Char code = 0xD1 (209), Char = */
		0xD2,		0xC0, 0x80, 0xFF, 0xFF, 0x80, 0xC0,	/* Char code = 0xD2 (210), Char = */
		0xD3,		0xF0, 0xF9, 0x09, 0x09, 0xFF, 0xFE,	/* Char code = 0xD3 (211), Char = */
		0xD6,		0xFE, 0xFE, 0x02, 0xFE, 0xFE, 0x03,	/* Char code = 0xD6 (214), Char = */
		0xD7,		0xF0, 0xF8, 0x08, 0x08, 0xFF, 0xFF,	/* Char code = 0xD7 (215), Char = */
		0xD9,		0xFE, 0x02, 0x3E, 0x02, 0xFF, 0x03,	/* Char code = 0xD9 (217), Char = */
		//0xDC,		0xFF, 0xFF, 0x11, 0x11, 0x1F, 0x0E,	/* Char code = 0xDC (220), Char = */
		0xDE,		0xFF, 0x08, 0x7E, 0x81, 0xFF, 0x7E,	/* Char code = 0xDE (222), Char = */
		0xDF,		0x71, 0xF9, 0x8A, 0x8C, 0xFF, 0xFF,	/* Char code = 0xDF (223), Char = */
		0x00 // END
	};
	
	
	
	
////////////////////////////////////////////////////////////////////////// forward definitions
static const uint8_t* getFontCharAddr(uint8_t symbol);
static void outRaster(uint8_t vPos, uint8_t width, const uint8_t raster[], matrixUpdateType update);
static matrixUpdateType effectExec(void);
	
//////////////////////////////////////////////////////////////////////////

void matrixExec(uint8_t ticks){
	if (!ticks)  {
		memset(&matrix, 0, sizeof(matrix));
		matrix.update = muNoUpdate;
		matrix.intensity = 0x07;
	}// if ticks
	
	//  -    .    -  
	if (matrix.update == muNoUpdate) return;
	if (matrix.update == muImmediate) matrix.update = muNoUpdate;
	if (matrix.update > muImmediate) { 
		matrix.update = effectExec();
		if (effectsData.pause) return;
	} // if
	
	//  
	uint8_t i, j;
	// init
	matrixSetCsLevel(0);
	for (i=0;i<4;i++) matrixSetReg( MREG_TESTMODE,	0x00 );
	matrixSetCsLevel(1);
	matrixSetCsLevel(0);
	for (i=0;i<4;i++) matrixSetReg( MREG_STANDBY,	0x01 );
	matrixSetCsLevel(1);
	matrixSetCsLevel(0);
	for (i=0;i<4;i++) matrixSetReg( MREG_SCANLIMIT,	0x07 );
	matrixSetCsLevel(1);
	matrixSetCsLevel(0);
	for (i=0;i<4;i++) matrixSetReg( MREG_DECODING,	0x00 );
	matrixSetCsLevel(1);
	matrixSetCsLevel(0);
	for (i=0;i<4;i++) matrixSetReg( MREG_INTENSITY,	matrix.intensity );
	matrixSetCsLevel(1);
	
	uint8_t vMask = 128;
	for(j=1; j<=8; j++) {
		uint8_t mask = 1;
		uint8_t val = 0;
		matrixSetCsLevel(0);
		i = 32;
		do {
			i--;
			if (matrix.videoRam[i] & vMask) val |= mask;
			mask <<= 1;
			if (!mask) {
				matrixSetReg(j, val);
				mask = 1;
				val = 0;
			} // if !mask
		} while(i);
		matrixSetCsLevel(1);
		vMask >>= 1;
	}	// for j
}

void matrixClear (matrixUpdateType update){
	uint8_t i;
	for (i=0;i<32; i++) {
		matrix.shadowRam[i] = 0;
		if (update == muImmediate) matrix.videoRam[i] = 0;
	}
	if (update != muNoUpdate) matrix.update = update;
	//if (matrix.update == muNoUpdate) matrix.update = update;
}

void matrixOutSymbol(uint8_t vPos, uint8_t symbol, matrixUpdateType update) {
	outRaster(vPos, MAINFONT_WIDTH, getFontCharAddr(symbol), update);		
}


void matrixOutImage(uint8_t vPos, const uint8_t image[], matrixUpdateType update){
	outRaster(vPos, pgm_read_byte(image), image+1, update);
}

void matrixSetIntensity(uint8_t intensity){
	matrix.intensity = (intensity>0x0F) ? 0x0F : intensity;
	//if (matrix.update == muNoUpdate) matrix.update = muImmediate;
}

// void matrixSetPixel(uint8_t col, uint8_t row, uint8_t pixel){
// 	uint8_t mask = 1 << row;
// 	if (pixel) matrix.videoRam[col] |= mask;
// 	else matrix.videoRam[col] &= ~mask;
// 	matrix.shadowRam[col] = matrix.videoRam[col];
// 	matrix.update = muImmediate;
// }

void matrixInitRollText(uint8_t textBuf[], uint8_t tailStopPos){
	effectsData.buff = textBuf;
	effectsData.pause = 0;
	effectsData.colCount = 0;
	effectsData.tailStop = tailStopPos;
	effectsData.tail = 31;
	matrix.update = muRollText;
}

uint8_t matrixIsEffectExecute(void){
	return matrix.update != muNoUpdate;
}

void matrixOutPStr(uint8_t vPos, const uint8_t str[], uint8_t maxSymbols, matrixUpdateType update){
	uint8_t ch;
	do {
		ch = pgm_read_byte(str++);
		if (ch) matrixOutSymbol(vPos, ch, update);
		vPos += 1 + MAINFONT_WIDTH;
		maxSymbols--;
	} while (ch && maxSymbols); 
}

////////////////////////////////////////////////////////////////////////// static
																								
static const uint8_t* getFontCharAddr(uint8_t symbol){			
	uint8_t fontStep = MAINFONT_WIDTH+1;									
	const uint8_t* pSymb = mainFont;
	uint8_t ch;
	do {
		ch = pgm_read_byte(pSymb);
		if (ch == symbol) return pSymb+1;
		pSymb += fontStep;
	} while (ch && ch<symbol);
	return mainFont+1;	
}

static void outRaster(uint8_t vPos, uint8_t width, const uint8_t raster[], matrixUpdateType update) {
	//uint8_t* pImg = (uint8_t*)((uint16_t)symbol * width + FONT_DATA_SHIFT + font);
	while(width  &&  vPos<32) {
		uint8_t column = pgm_read_byte(raster++);
		matrix.shadowRam[vPos] = column;
		if (update == muImmediate) matrix.videoRam[vPos] = matrix.shadowRam[vPos];
		vPos++;
		width--;
	}
	if (update == muRandom) { 
		update = FIRST_MATRIX_UPDATE_MODE + getRand(LAST_MATRIX_UPDATE_MODE - FIRST_MATRIX_UPDATE_MODE +1); 
		} 
	if (update == muNoUpdate) return;
	//if (matrix.update != muNoUpdate) return;
	matrix.update = update;
	switch (update) {
		case muNoUpdate:
		case muImmediate:
		case muRollText:
		case muRandom:
			break;
		case muRollLeft:
		case muRollRight:
		case muWaterFall:
		case muRollUp:
		case muRollDown:
			effectsData.pause = 0;
			effectsData.tail = 0;
			break;
	}	//switch (update)
}

static matrixUpdateType effectExec(void){
	// pause
	if (effectsData.pause) {
		effectsData.pause--;
		return matrix.update;
	}
	uint8_t i;
	switch (matrix.update) {
		case muRollText:		//     
			if (*effectsData.buff == 0 && effectsData.tail == effectsData.tailStop) {
			  vRam2shRam:
				memcpy(matrix.shadowRam, matrix.videoRam,32);
				return muNoUpdate;
			}
			effectsData.pause = ROLLING_DELAY-1;
			for (i = 0; i<31; i++) matrix.videoRam[i] = matrix.videoRam[i+1];
			matrix.videoRam[31] = 0;
			if (!effectsData.colCount) {
				if (*effectsData.buff) {
					effectsData.fontpos = getFontCharAddr(*effectsData.buff);
					effectsData.colCount = MAINFONT_WIDTH+1;					   
					effectsData.buff++;
				} else effectsData.tail--;
			} // if !colCount
			if (effectsData.colCount) {
				if (effectsData.colCount == MAINFONT_WIDTH+1) matrix.videoRam[31] = 0;  
				else matrix.videoRam[31] = pgm_read_byte(effectsData.fontpos++);
				effectsData.colCount--;
				effectsData.tail = 32;
			}
			return matrix.update;
			//break;
		case muRollLeft: //   
		case muRollRight: //   
			#define	EMPTY_SIZE	8
			if (effectsData.tail == 32+EMPTY_SIZE) return muNoUpdate;
			effectsData.pause = ROLLING_DELAY-1;
			if (matrix.update == muRollLeft) {
				for (i = 0; i<31; i++) matrix.videoRam[i] = matrix.videoRam[i+1];
				if (effectsData.tail>=EMPTY_SIZE) matrix.videoRam[31] = matrix.shadowRam[effectsData.tail-EMPTY_SIZE];	
				else matrix.videoRam[31] = 0;
			} else {
				for (i = 31; i; i--) matrix.videoRam[i] = matrix.videoRam[i-1];
				if (effectsData.tail>=EMPTY_SIZE) matrix.videoRam[0] = matrix.shadowRam[31+EMPTY_SIZE-effectsData.tail];
				else matrix.videoRam[0] = 0;
			}
			effectsData.tail++;
			return matrix.update;
			#undef EMPTY_SIZE
			//break;
		case muWaterFall:
			if (effectsData.tail == 32+20) goto vRam2shRam;// return muNoUpdate;
			effectsData.pause = WATERFALL_DELAY-1;
			for (i = 0; i<32; i++) {
				int8_t phase = effectsData.tail - i;
				if (phase<0) continue;
				if (phase<20) {
					matrix.videoRam[i] >>= 1;
					if (phase>11) {
						uint8_t mask = 1 << (phase-12);
						if (matrix.shadowRam[i] & mask) matrix.videoRam[i] |= 0x80;
					} // if phase >11
				} // if phase < 20
			}	//for i
			effectsData.tail++;
			return matrix.update;
			//break;
		case muRollUp:
		case muRollDown:
			if (effectsData.tail == 20) goto vRam2shRam;// return muNoUpdate;
			effectsData.pause = VROLLING_DELAY-1;
			if (effectsData.tail == 12) effectsData.tailStop = (matrix.update == muRollUp) ? 0x80 : 0x01;
			for (i = 0; i<32; i++) {
				if (matrix.update == muRollUp) matrix.videoRam[i] <<= 1;
				else matrix.videoRam[i] >>= 1;
				if (effectsData.tail>11) {
					if (matrix.shadowRam[i] & effectsData.tailStop) matrix.videoRam[i] |= (matrix.update == muRollUp) ? 0x01 : 0x80;
				} // if phase >11
			}	//for i
			if (matrix.update == muRollUp) effectsData.tailStop >>= 1;
			else effectsData.tailStop <<= 1;
			effectsData.tail++;
			return matrix.update;
			//break;
		default:
			return matrix.update;
			//break;
	} // switch
}

