/*
 * rtc_ds3231.c
 *
 * Created: 27.11.2018 21:46:47
 *  Author: GoldenAndy
 */ 

#include "defines.h"
#include "hal.h"
#include "rtc_ds3231.h"
#include "../_common/i2c_dumb_master/i2c_mstr.h"
#include <avr/pgmspace.h>
#include <string.h>

#define RTCAddr				0xD0



static uint16_t i2cFailTimeCounter;
//static uint8_t i2cBusyFlags = 0;
static uint8_t errorRtcFlag = 0;
// #ifndef DISABLE_AUTO_DAYLIGHTSAVING
// static uint8_t dlsFlag;
// #endif
timeStampType timeStamp; 

#ifndef DISABLE_AUTO_DAYLIGHTSAVING
uint8_t isDLS(timeStampType* stamp, uint8_t dH){
	//dH -  , 0        1      
	//if (!dlsFlag) return 0;
	if (stamp->month > 0x03 && stamp->month < 0x10 ) return 1;
	if (stamp->month < 0x03 || stamp->month > 0x10) return 0;
	// month == 0x03 or 0x10
	uint8_t day31 = calculateDay(31,bcd2int(stamp->month),bcd2int(stamp->year));
	if (day31==7) day31=0;
	uint8_t dayX = int2bcd(31-day31);
	if (stamp->month == 0x03 ) {  // 
		if (stamp->date < dayX) return 0;
		else if (stamp->date > dayX) return 1;
		if (stamp->hours < 0x03) return 0;
		else return 1;
	}
	if (stamp->month == 0x10 ) {  // 
		if (stamp->date > dayX) return 0;
		else if (stamp->date < dayX) return 1;
		if (stamp->hours >= 0x04+dH) return 0;
		else return 1;
	}
	return 0; //    ,    
}

void makeDayLightSaving(void){
	//      timeStamp     
	if (!isDLS(&timeStamp, 0)) return; 
	timeStamp.hours = incBcd(timeStamp.hours);
	if (timeStamp.hours>0x23) {
		timeStamp.hours = 0x00;
		if (timeStamp.day<7) timeStamp.day++;
		else timeStamp.day = 1;
		uint8_t lastD = getLastMonthDate(bcd2int(timeStamp.month),bcd2int(timeStamp.year));
		timeStamp.date = incBcd(timeStamp.date);
		if (timeStamp.date > int2bcd(lastD)) {
			timeStamp.date = 0x01;
			timeStamp.month = incBcd(timeStamp.month);
			//    ,        ...
		} // date>lastdate
	}	// hours>23
}
#endif


void initRTC(void){
	i2cFailTimeCounter = 0;
	timeStamp.seconds = 0xFF;
	
	uint8_t iicBuf[2];
	// 0x0E bits 7..0 :  ~EOSC BBSQW   CONV	  RS2  RS1   INTCN   A2IE  A1IE
	//                    80h	40h		20h	  10h  08h	  04h	  02h	01h
	//						0	0		0	   00=1Hz    0=wave	  0	    0
	iicBuf[0]	= 0x00;
	// 0x0F bits 7..0 :  OSF   0   0   0   EN32KHZ   BSY  A2F  A1F
	//                   80h  40h 20h 10h    08h	 04h  02h  01h
	//				 	   0   0   0   0      0		  0	   0    0
	iicBuf[1]	= 0x00;
	errorRtcFlag = 0 == iic_writeAddrWriteData(RTCAddr, 0x0E, iicBuf, 2);
//	i2cBusyFlags = 0;
}

const uint8_t monthData[12] PROGMEM	 = { 0, 3, 3, 6, 1, 4, 6, 2, 5, 0, 3, 5 };

uint8_t calculateDay(uint8_t date, uint8_t month, uint8_t shortYear){
	//     ,     .  = 1..7 (..).
	// !   - BCD!!!
	//    21 !
	date = bcd2int(date);
	month = bcd2int(month);
	shortYear = bcd2int(shortYear);
	uint8_t result = ( pgm_read_byte(&monthData[month-1]) + date ) % 7;
	result += ( shortYear % 28 ) + ( shortYear / 4 );
	if (!(shortYear & 0x03) && month < 3 ) result += 6;
	result = 1 + ( 5 + result ) % 7;
	return result;
}


static void RTCRead(void){
	uint8_t readBuffer[7];
	errorRtcFlag = 0 == iic_writeAddrReadData(RTCAddr, 0x00, readBuffer, 7);
	if (!errorRtcFlag) memcpy((void*)&timeStamp, readBuffer, 7);
}


void writeTime(timeStampType* data) {
	data->hours &= 0x3F; //  6,7 - .  24  
	//data[3] = 0x07 & ((writeBuffer[3]==0) ? 7 : writeBuffer[3]); //   =0 -    7,    0..6 (..),  1..7 (..)
	data->day = calculateDay(data->date,data->month,data->year);
	data->month |= 0x80;   // 7 - .  21 
	iic_writeAddrWriteData(RTCAddr, 0x00, (uint8_t*)data, 7);
	RTCRead();
}


void execRtc(uint8_t needRead){
	//     
	if (needRead) {
		RTCRead();
		//        
		timeStamp.month	&= 0x7F;	//7    . 0 = XX, 1=XXI .    , 
		if (errorRtcFlag) timeStamp.seconds = 0xFF;
		timeStamp.day = calculateDay(timeStamp.date, timeStamp.month, timeStamp.year);
	}//if (needRead)

	//   
	if ( timeStamp.seconds < 0x60 ) { //   ...
		i2cFailTimeCounter = 0;
		//    -    -  
		//static uint8_t prevMinute = 0xFF;
		//if (prevMinute != timeStamp.minutes) {
		//	prevMinute = timeStamp.minutes;
		//	//     
		//}
	} //    ... 
	
	#define XSYS (10*SYSTEMCLOCK_FREQ)
	//#define XDIV (1+XSYS/256)
	if (++i2cFailTimeCounter > XSYS) {
		// 
		initRTC();
	}  
}

uint8_t bcd2int(uint8_t bcdValue){
	return 10 * ( bcdValue >> 4 ) + ( bcdValue & 0x0F );
}

uint8_t int2bcd(uint8_t intValue){
	return (intValue / 10) * 16 + intValue % 10;
}

uint8_t incBcd(uint8_t bcd){
	if ((bcd & 0x0F) > 8) return bcd + 7;
	return bcd+1;
}

uint8_t decBcd(uint8_t bcd){
	if (bcd & 0x0F) return bcd-1;
	return bcd-7;
}

uint8_t getLastMonthDate(uint8_t month, uint8_t shortYear){
	//        .
	//   -  BCD!!!
	if (month == 2) {
		if (shortYear & 0x03) return 28; else return 29;
	}
	if ( month == 4 || month == 6 || month == 9 || month == 11 ) return 30;
	return 31;
}

// #ifndef DISABLE_AUTO_DAYLIGHTSAVING
// void setDayLightSaveFlag(uint8_t flag){
// 	dlsFlag = flag;
// }
// #endif

#undef RTCAddr		

#undef i2f_needWriteData

#undef XSYS