/*
 * soundPatternPlay.c
 *
 *  Author: GoldenAndy
 */ 

#include "soundPatternPlay.h"

#if !defined(_SIMPLE_SOUND_DURATION_)
	#error "_SIMPLE_SOUND_DURATION_ must be defined"
#elif (_SIMPLE_SOUND_DURATION_<1 || _SIMPLE_SOUND_DURATION_>255)
	#error "_SIMPLE_SOUND_DURATION_ value must be between 1 and 255"	
#endif

static soundPatternType soundBar = 0;

void (*setSoundOnProc)(void) = 0;			//   
void (*setSoundOffProc)(void) = 0;			//   

uint8_t soundPatternIsEmpty(void) {
	return soundBar == 0;
}

void setSoundPattern(soundPatternType soundPattern){
	soundBar = soundPattern;
}

void execSoundPattern(uint8_t ticks, uint8_t init){
	if (!setSoundOnProc || !setSoundOffProc) return;
	static uint8_t soundCounter = 0;
	if (init) {
		soundCounter = 0;
		soundBar = 0;
		(*setSoundOffProc)();
		return;
	}
	if (soundCounter || soundBar)  {
		while (soundCounter && ticks) {
			soundCounter--;	 
			ticks--; 
		}
		if (!soundCounter) {
			if (soundBar) soundCounter = _SIMPLE_SOUND_DURATION_;
			if (soundBar & 0x01) (*setSoundOnProc)();
			else (*setSoundOffProc)();
			soundBar >>= 1;
		} // if (!soundCounter)
	} // if (soundCounter || soundBar)
}

void setSoundPatternControls(void (*fSoundOn)(void), void (*fSoundOff)(void)) {
	setSoundOnProc = fSoundOn;
	setSoundOffProc = fSoundOff;
}
