#ifndef _ANTI_BOUNCE_
#define _ANTI_BOUNCE_ 1

#include <stdint.h>
#include "defines.h"

/*****************************************
 *	   
 *
 *	       . 
 *	       .
 *
 *	  :
 *		keyAntiBounce -	 ,       .
 *							   bounceCounterType   .
 *							
 *		keyProcessingInit -     . 
 *							,      _KEY_BOUNCE_INDIVIDUAL_TIMINGS_  
 *							  .
 *
 *		keyProcessing -	   .      , 
 *							 0  
 *							    keyProcessingStruct
 *							  keyAntiBounce.
 *
 *		encoderProcessingInit -      .
 *							     _KEY_BOUNCE_INDIVIDUAL_TIMINGS_   
 *							 .
 *
 *		encoderProcessing	-   .         
 *							  .
 *							    encoderProcessingStruct,   
 *							keyProcessing  keyProcessingInit							
 *							       _KEY_EVENT_ROTATE_,   
 *							 
 *		encoderGetRotates   -     ,   
 *							
 *
 *	 :
 *	
 *	_KEY_BOUNCE_8BIT_
 *	
 *	_KEY_BOUNCE_16BIT_		   ,      
 *
 *	_KEY_BOUNCE_INDIVIDUAL_TIMINGS_ -      
 *	
 *	_KEY_LONG_PRESS_TIME_	-    ,    .
 *	_KEY_REPEAT_TIME_		-    ,    .
 *	_KEY_BOUNCE_TIME_		-     .
 *
 *	       ,    
 *
 *****************************************/


//=====================================================================================================================================
// keyAntiBounce -     . 
//      ,     -

//        ,   ,     
//   ..    /      8-   16-,  
//   _KEY_BOUNCE_8BIT_  _KEY_BOUNCE_16BIT_.       .

// bounceCounterType -     .

#if defined(_KEY_BOUNCE_8BIT_) && !defined(_KEY_BOUNCE_16BIT_)
	typedef uint8_t bounceCounterType;
#elif defined(_KEY_BOUNCE_16BIT_) && !defined(_KEY_BOUNCE_8BIT_)
	typedef uint16_t bounceCounterType;
#else
	#error The key hysteresis width must be declared.  	_KEY_BOUNCE_8BIT_ or _KEY_BOUNCE_16BIT_ 
#endif

	/********************
	 *	@brief	keyAntiBounce -	  
	 *	@param	keyCounter			      
	 *	@param	isPhysicallyPressed    
	 *	@param	keyBounceValue		       
	 *	@return	1 -  , 0 -  
	 ********************/
uint8_t keyAntiBounce (bounceCounterType * keyCounter, uint8_t isPhysicallyPressed, bounceCounterType keyBounceValue);




//=====================================================================================================================================
//=====================================================================================================================================
// keyProcessing -    .      ,  0  

// keyProcessingStruct -         .
typedef struct {
	uint8_t processingFlags;
	uint8_t previousState;
	uint16_t pressCounter;
	bounceCounterType bounceCounter;
	#ifdef _KEY_BOUNCE_INDIVIDUAL_TIMINGS_
	bounceCounterType bounceTime;
	uint16_t longPressTime;
	uint16_t repeatPressTime;
	#endif
} keyProcessingStruct;

/********************
 *	name		:	keyProcessing
 *	description	:	   .      ,  0  .
 *	arguments	:	keyStruct			->     
 *					isPhysicallyPressed ->   
 *					forceReset			->          
 *	result		:	   
 ********************/
uint8_t keyProcessing(keyProcessingStruct * keyStruct, uint8_t isPhysicallyPressed, uint8_t forceReset);


//       
#define _KEY_FLAG_SHORTPRESS_	1
#define _KEY_FLAG_LONGPRESS_	2
#define _KEY_FLAG_REPEAT_		4
#define _KEY_FLAG_EVENTS_		8

// .       
#define _KEY_ENCODER_NO_PRESS_ON_ROTATE_	16

// ,     
#define _KEY_EVENT_SHORTPRESS_	1
#define _KEY_EVENT_LONGPRESS_	2
#define _KEY_EVENT_REPEAT_		4
#define _KEY_EVENT_ON_PRESS_	8
#define _KEY_EVENT_ON_RELEASE_	16
#define _KEY_EVENT_ROTATE_		32


/************************************
 *	              . 
 *	      .
 *	0						-    .
 *	_KEY_FLAG_SHORTPRESS_	-    ,      .
 * 	_KEY_FLAG_LONGPRESS_	-    ,      
 *								 ,       .
 *	_KEY_FLAG_SHORTPRESS_ | _KEY_FLAG_LONGPRESS_	-     .     
 *								 ,        ,
 *								,        ,   
 *								 ,       .
 *	_KEY_FLAG_SHORTPRESS_ | _KEY_FLAG_REPEAT_		-      .
 *								       , ,   
 *								           
 *  _KEY_FLAG_REPEAT_		-    _KEY_FLAG_SHORTPRESS_ | _KEY_FLAG_REPEAT_
 *	     _KEY_FLAG_LONGPRESS_  _KEY_FLAG_REPEAT_.   
 *								     - _KEY_FLAG_SHORTPRESS_
 *	_KEY_FLAG_EVENTS_		-        .
 *								 ,      ,     
 *								  .
 *	
 *	_KEY_ENCODER_NO_PRESS_ON_ROTATE_ -    .       .
 *
 *	      :
 *	0	-   
 *	_KEY_EVENT_SHORTPRESS_ -    
 *	_KEY_EVENT_LONGPRESS_ -    
 *	(_KEY_EVENT_SHORTPRESS_ | _KEY_EVENT_REPEAT_) -   ,       .
 *	_KEY_EVENT_ON_PRESS_  -     
 *	_KEY_EVENT_ON_RELEASE_ -     
 *	_KEY_EVENT_ROTATE	-    
 *
 *
 *	       
 *
 *	    : 
 *		    
 *		   
 *	    _KEY_BOUNCE_INDIVIDUAL_TIMINGS_
 *	    -         
 *	 ,     4  ( 2  uint16_t ).
 *	    -     :
 *	_KEY_LONG_PRESS_TIME_	-    ,    .
 *	_KEY_REPEAT_TIME_		-    ,    .
 *	_KEY_BOUNCE_TIME_		-    .
 *	          .
 *
 ************************************/



//=====================================================================================================================================
//=====================================================================================================================================
// keyProcessingInit -     . 
//     _KEY_BOUNCE_INDIVIDUAL_TIMINGS_    .

	//   
	#if !defined(_KEY_BOUNCE_INDIVIDUAL_TIMINGS_) && ( !defined(_KEY_LONG_PRESS_TIME_) || !defined(_KEY_REPEAT_TIME_) || !defined(_KEY_BOUNCE_TIME_) )
	#error Define _KEY_BOUNCE_INDIVIDUAL_TIMINGS_ or defines _KEY_BOUNCE_TIME_, _KEY_LONG_PRESS_TIME_ and _KEY_REPEAT_TIME_ must be defined
	#endif

	#ifdef	_KEY_BOUNCE_INDIVIDUAL_TIMINGS_
		/********************
		 *	name		:	keyProcessingInit
		 *	description	:	  .     . 
		 *	arguments	:	keyStruct	->     
		 *					keyMode		->    
		 *					longTime	->   									   
		 *					repeatTime	->   
		 *					bounceTime	->  
		 *	result		:	none
		 ********************/
		void keyProcessingInit(keyProcessingStruct * keyStruct, uint8_t keyMode, uint16_t longTime, uint16_t repeatTime, bounceCounterType bounceTime);
	#else	//_KEY_BOUNCE_INDIVIDUAL_TIMINGS_ is not defined
		/********************
		 *	name		:	keyProcessingInit
		 *	description	:	  .     .
		 *	arguments	:	keyStruct	->     
		 *					keyMode		->    
		 *	result		:	none
		 ********************/
		void keyProcessingInit(keyProcessingStruct * keyStruct, uint8_t keyMode);
	#endif //ifdef _KEY_BOUNCE_INDIVIDUAL_TIMINGS_



//=====================================================================================================================================
//=====================================================================================================================================
// encoderProcessingInit -     .
//     _KEY_BOUNCE_INDIVIDUAL_TIMINGS_    .


// encoderProcessingStruct -        
typedef struct {
	bounceCounterType	chA;
	bounceCounterType	chB;
	bounceCounterType	rotaryBounce;
	uint8_t				prevEncValue;
	keyProcessingStruct btn;
	int8_t				rotates;
} encoderProcessingStruct;

	#ifdef	_KEY_BOUNCE_INDIVIDUAL_TIMINGS_
		void encoderProcessingInit(encoderProcessingStruct * encStruct, 
									bounceCounterType rotaryBounceTime,
									uint8_t keyMode, uint16_t longTime, uint16_t repeatTime, bounceCounterType keyBounceTime);
	#else	//_KEY_BOUNCE_INDIVIDUAL_TIMINGS_ is not defined
		void encoderProcessingInit(encoderProcessingStruct * encStruct, uint8_t keyMode);
	#endif								

//=====================================================================================================================================
//=====================================================================================================================================
// encoderProcessing -    .         ,  0  
//
uint8_t encoderProcessing(encoderProcessingStruct * encStruct, uint8_t isPhysicallyPressed, uint8_t isPhysicallyChA, uint8_t isPhysicallyChB, uint8_t forceReset);

//=====================================================================================================================================
//=====================================================================================================================================
// encoderGetRotates -     ,    
int8_t encoderGetRotates(encoderProcessingStruct * encStruct);






#endif //_ANTI_BOUNCE_